"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EdmTimeOfDay = void 0;
var TimeOfDayRegex = new RegExp(/^\d{2}:\d{2}(:\d{2}(\.\d{1,12})?)?$/);
var partLengths = [2, 2, 2];
var EdmTimeOfDay = (function () {
    function EdmTimeOfDay(source) {
        source instanceof Date ? this.parseDate(source) : this.parseString(source.toString());
    }
    EdmTimeOfDay.create = function (source) {
        if (source === void 0) { source = '00:00'; }
        var result = source instanceof EdmTimeOfDay
            ? source
            : source === null
                ? null
                : new EdmTimeOfDay(source);
        return result;
    };
    Object.defineProperty(EdmTimeOfDay.prototype, "hours", {
        get: function () {
            return this._hours;
        },
        set: function (value) {
            this.updateTimeOfDay(value, this._minutes, this._seconds, this._fractionalSeconds);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(EdmTimeOfDay.prototype, "minutes", {
        get: function () {
            return this._minutes;
        },
        set: function (value) {
            this.updateTimeOfDay(this._hours, value, this._seconds, this._fractionalSeconds);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(EdmTimeOfDay.prototype, "seconds", {
        get: function () {
            return this._seconds;
        },
        set: function (value) {
            this.updateTimeOfDay(this._hours, this._minutes, value, this._fractionalSeconds);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(EdmTimeOfDay.prototype, "fractionalSeconds", {
        get: function () {
            return this._fractionalSeconds;
        },
        set: function (value) {
            var seconds = Math.trunc(value);
            this._seconds += seconds;
            value -= seconds;
            this.updateTimeOfDay(this._hours, this._minutes, this._seconds, value);
        },
        enumerable: false,
        configurable: true
    });
    EdmTimeOfDay.prototype.toString = function () {
        var result = this.formatTimeOfDay(this._hours, this._minutes, this._seconds, this._fractionalSeconds);
        return result;
    };
    EdmTimeOfDay.prototype.formatTimeOfDay = function (hours, minutes, seconds, fractionalSeconds) {
        var negative = hours < 0 || minutes < 0 || seconds < 0 || fractionalSeconds < 0;
        var sign = negative ? '-' : '';
        var result = sign + [hours, minutes, seconds]
            .map(function (x, i) { return Math.abs(x).toString().padStart(partLengths[i], '0'); })
            .join(':')
            + (fractionalSeconds ? Math.abs(fractionalSeconds).toString().substring(1) : '.').padEnd(4, '0');
        return result;
    };
    EdmTimeOfDay.prototype.parseDate = function (source) {
        var hms = source.toTimeString().split(' ').shift();
        var fs = source.getMilliseconds() / 1000;
        this._fractionalSeconds = fs;
        this.parseHourMinuteSecond(hms);
    };
    EdmTimeOfDay.prototype.parseString = function (source) {
        if (!TimeOfDayRegex.test(source)) {
            throw new TypeError("'".concat(source, "' is not a valid ").concat(EdmTimeOfDay.name));
        }
        var _a = source.split('.'), hms = _a[0], ms = _a[1];
        this._fractionalSeconds = Number('.' + (ms !== null && ms !== void 0 ? ms : 0));
        this.parseHourMinuteSecond(hms);
    };
    EdmTimeOfDay.prototype.parseHourMinuteSecond = function (source) {
        var _a = source.split(':').map(function (x) { return Number(x); }), hours = _a[0], minutes = _a[1], seconds = _a[2];
        this.updateTimeOfDay(hours, minutes, seconds !== null && seconds !== void 0 ? seconds : 0, this._fractionalSeconds);
    };
    EdmTimeOfDay.prototype.updateTimeOfDay = function (hours, minutes, seconds, fractionalSeconds) {
        var _a, _b;
        var totalSeconds = ((hours * 60 + minutes) * 60) + seconds;
        var secondsLeft = totalSeconds;
        _a = [3600, 60, 1].map(function (x) {
            var result = Math.trunc(secondsLeft / x);
            secondsLeft -= result * x;
            return result;
        }), hours = _a[0], minutes = _a[1], seconds = _a[2];
        if (totalSeconds + fractionalSeconds < 0 || hours >= 24) {
            throw new TypeError("'".concat(this.formatTimeOfDay(hours, minutes, seconds, fractionalSeconds), "' is not a valid ").concat(EdmTimeOfDay.name));
        }
        _b = [hours, minutes, seconds, fractionalSeconds], this._hours = _b[0], this._minutes = _b[1], this._seconds = _b[2], this._fractionalSeconds = _b[3];
    };
    EdmTimeOfDay.MinValue = new EdmTimeOfDay('00:00');
    EdmTimeOfDay.MaxValue = new EdmTimeOfDay('23:59:59.999999999999');
    return EdmTimeOfDay;
}());
exports.EdmTimeOfDay = EdmTimeOfDay;
//# sourceMappingURL=edm-timeofday.js.map