"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utilities = void 0;
var breeze_client_1 = require("breeze-client");
var ts_odatajs_1 = require("ts-odatajs");
var class_registry_1 = require("./class-registry");
var odata_error_1 = require("./odata-error");
var Utilities;
(function (Utilities) {
    Utilities.dataTypeMap = {
        bool: breeze_client_1.DataType.Boolean,
        float: breeze_client_1.DataType.Double,
        int: breeze_client_1.DataType.Int64,
        sbyte: breeze_client_1.DataType.Byte
    };
    function adaptMetadata(metadata) {
        if (metadata.version !== '4.0') {
            return metadata;
        }
        var metadataAdapters = class_registry_1.ClassRegistry.MetadataAdapters.get();
        var csdlMetadata = metadata.dataServices;
        metadataAdapters.forEach(function (a) {
            ts_odatajs_1.oData.utils.forEachSchema(csdlMetadata, a.adapt.bind(a));
        });
        return metadata;
    }
    Utilities.adaptMetadata = adaptMetadata;
    function getDataType(key) {
        var result = Utilities.dataTypeMap[key];
        if (result) {
            return result;
        }
        var enumName = breeze_client_1.DataType.getNames().find(function (n) { return n.toLowerCase() === key.toLowerCase(); });
        result = !!enumName ? breeze_client_1.DataType[enumName] : breeze_client_1.DataType.String;
        return result;
    }
    Utilities.getDataType = getDataType;
    function getEdmTypeFromTypeName(metadata, typeName) {
        var bindingTypeName = ts_odatajs_1.oData.utils.getCollectionType(typeName) || typeName;
        var entityType = ts_odatajs_1.oData.utils.lookupEntityType(bindingTypeName, metadata);
        var complexType = ts_odatajs_1.oData.utils.lookupComplexType(bindingTypeName, metadata);
        return entityType || complexType;
    }
    Utilities.getEdmTypeFromTypeName = getEdmTypeFromTypeName;
    function adaptComplexType(metadataStore, complexType) {
        return adaptStructuralType(metadataStore, complexType);
    }
    Utilities.adaptComplexType = adaptComplexType;
    function adaptEntityType(metadataStore, entityType) {
        return adaptStructuralType(metadataStore, entityType);
    }
    Utilities.adaptEntityType = adaptEntityType;
    function adaptStructuralType(metadataStore, structuralType) {
        var result = metadataStore.getStructuralType(structuralType.name, true);
        return result;
    }
    Utilities.adaptStructuralType = adaptStructuralType;
    function getActions(metadata, metadataStore) {
        var result = getInvokableEntries(metadata, metadataStore, function (s) { return s.action; });
        return result;
    }
    Utilities.getActions = getActions;
    function getFunctions(metadata, metadataStore) {
        var result = getInvokableEntries(metadata, metadataStore, function (s) { return s.function; });
        return result;
    }
    Utilities.getFunctions = getFunctions;
    function getInvokableEntries(metadata, metadataStore, accessor) {
        var result = [];
        metadata.dataServices.schema.forEach(function (s) {
            var namespace = s.namespace;
            var items = accessor(s);
            if (!items) {
                return;
            }
            var entries = items.map(function (config) {
                var entry = {
                    config: config,
                    namespace: namespace,
                    url: getInvokableUrl(metadata, metadataStore, config, namespace)
                };
                return entry;
            });
            result = result.concat(entries);
        });
        return result;
    }
    Utilities.getInvokableEntries = getInvokableEntries;
    function getInvokableUrl(metadata, metadataStore, config, namespace) {
        var isBound = Boolean(config.isBound);
        var boundPart = '';
        if (isBound) {
            var bindingParameter = config.parameter[0];
            var bindingEdmType = getEdmTypeFromTypeName(metadata, bindingParameter.type);
            var breezeType = adaptEntityType(metadataStore, bindingEdmType);
            if (breezeType) {
                boundPart = "".concat(breezeType.defaultResourceName, "/");
            }
        }
        var url = "".concat(boundPart).concat(namespace, ".").concat(config.name);
        return url;
    }
    Utilities.getInvokableUrl = getInvokableUrl;
    function parseValue(dataType, value) {
        var result = dataType.parseRawValue ? dataType.parseRawValue(value) : dataType.parse(value, 'string');
        return result;
    }
    Utilities.parseValue = parseValue;
    function createError(error, url) {
        var result = new odata_error_1.ODataError();
        var message = getMessage(error);
        result.message = message;
        var response = error && error.response;
        if (!response) {
            return result;
        }
        result.statusText = response.statusText;
        result.status = Number(response.statusCode || '0');
        if (url) {
            result.url = url;
        }
        result.body = response.body;
        if (response.body) {
            try {
                var body = JSON.parse(response.body);
                result.body = body;
                if (body['odata.error']) {
                    body = body['odata.error'];
                }
                var msgs = [];
                do {
                    msgs.push(getMessage(body));
                    body = body.error || body.innererror || body.internalexception;
                } while (body);
                msgs = __spreadArray([result.message], msgs, true).filter(function (m) { return !!m; });
                if (msgs.length) {
                    result.message = msgs.join('; ');
                }
            }
            catch (e) { }
        }
        if (!result.message) {
            result.message = response.statusText || null;
        }
        breeze_client_1.AbstractDataServiceAdapter._catchNoConnectionError(result);
        return result;
    }
    Utilities.createError = createError;
    function getMessage(body) {
        if (typeof body === 'string') {
            return body;
        }
        var messageKey = Object.keys(body)
            .find(function (k) { return k.toLowerCase() === 'message'; });
        if (!messageKey) {
            return '';
        }
        var msg = body[messageKey];
        return typeof msg === 'string' ? msg : msg.value;
    }
})(Utilities = exports.Utilities || (exports.Utilities = {}));
//# sourceMappingURL=utilities.js.map